@@ -126,7 +126,7 @@ sub show {
         $pass{path}     = $args->{'sorry.path'}     if exists($args->{'sorry.path'});
     }
 
-    throw XAO::E::DO::Web::Order "show - no path or template" unless exists($pass{template})
+    throw $self "show - no path or template" unless exists($pass{template})
                                                                   || exists($pass{path});
     foreach (keys %pass) { delete $pass{$_} if /^sorry/; }
     $self->_display(\%pass);
@@ -232,8 +232,7 @@ sub add () {
 
 =item save ()
 
-Changes status of order to 'Placed' and adds customer_id to order
-of orders.  Takes arguments:
+Changes status of order to 'Placed' and adds customer_id to the order.
 
     * 'template' or 'path'
     * 'sorry.template' or 'sorry.path' or sorry'
@@ -241,26 +240,19 @@ of orders.  Takes arguments:
 =cut
 
 sub save {
-    
-    #dprint "***\n"
-    #     . "*** XAO::DO::Web::Order::save() START\n"
-    #     . "***";
+    my $self=shift;
+    my $args=get_args(\@_);
 
-    my $self = shift;
-    my $args = get_args(\@_);
-
-    my %pass;
-    foreach (keys %$args) { $pass{$_} = $args->{$_} if $args->{$_}; }
+    my $pass=merge_refs($args);
 
     my $order_id = $args->{id} || '';
     my $order    = $self->_get_order($order_id);
     if ($order && $order->get('Products')->keys) { # XXX hardcoded for now
-        #dprint "    %% Found Order With Items";
-        # Note: get customer from clipboard - this requires IdentifyUser to have
-        # run previously in 'check' mode!
+        # Note: get customer from clipboard - this requires IdentifyUser
+        # to have run previously in 'check' mode!
         # XXX 'customer' is hardcoded for now
-        my $customer = $self->_get_customer('customer')
-                    || throw XAO::E::DO::Web::Order "save - no customer object";
+        my $customer = $self->_get_customer('customer') ||
+            throw $self "save - no customer object";
         # XXX 'customer_id', 'status' and 'Placed' are hardcoded for now
         $order->put('customer_id', $customer->get('customer_id'));
         $order->put('status',      'Placed');
@@ -276,20 +268,15 @@ sub save {
         );
     }
     else {
-        #dprint "    %% No Order";
-        $pass{template} = $args->{'sorry.template'} if exists($args->{'sorry.template'});
-        $pass{path}     = $args->{'sorry.path'}     if exists($args->{'sorry.path'});
+        $pass->{template}=$args->{'sorry.template'} if exists $args->{'sorry.template'};
+        $pass->{path}=$args->{'sorry.path'} if exists $args->{'sorry.path'};
     }
 
-    foreach (keys %pass) { delete $pass{$_} if /^sorry/; }
-    $self->_display(\%pass);
-
-    #dprint "***\n"
-    #     . "*** XAO::DO::Web::Order::save() STOP\n"
-    #     . "***";
+    $self->_display($pass);
 
     return 1;
 }
+
 ###############################################################################
 
 =item expand_items (%)
@@ -310,7 +297,7 @@ sub expand_items {
 
     my $page     = $self->object;
     my $order_id = $args->{id} || '';
-    my $order    = $self->_get_order($order_id) || throw XAO::E::DO::Web::Order "expand_items - no order";
+    my $order    = $self->_get_order($order_id) || throw $self "expand_items - no order";
     my $ilist = $order->get('Products');
     my $text  = '';
 
@@ -320,7 +307,7 @@ sub expand_items {
     $text .= $page->expand(\%hpass) if exists($hpass{template}) || exists($hpass{path});
 
     # XXX Later: group and order items in a deliberate way
-    throw XAO::E::DO::Web::Order "expand_items - no item path or template"
+    throw $self "expand_items - no item path or template"
       unless exists($args->{'item.template'}) || exists($args->{'item.path'});
     my $count = 1;
     foreach ($ilist->keys) {
@@ -361,7 +348,7 @@ sub expand_items {
 sub clear_items {
     #dprint "*** XAO::DO::Web::Order::clear_items() START";
     my ($self, $rh_cgi) = @_;
-    my $order  = $self->_get_order || throw XAO::E::DO::Web::Order "clear_items - no order";
+    my $order  = $self->_get_order || throw $self "clear_items - no order";
     my $oilist = $order->get('Products');
     $oilist->destroy;
     #dprint "*** XAO::DO::Web::Order::clear_items() STOP";
@@ -386,7 +373,7 @@ sub add_items {
 
     my ($self, $rh_cgi) = @_;
 
-    my $order  = $self->_get_order || throw XAO::E::DO::Web::Order "add_items - no order";
+    my $order  = $self->_get_order || throw $self "add_items - no order";
     my $oilist = $order->get('Products');
     my $ilist  = $self->odb->fetch('/Products'); # XXX hardcoded for now
 
@@ -394,7 +381,7 @@ sub add_items {
     while (my $id = $rh_cgi->{'item'.$count}) { # XXX hardcoded for now
 
         unless ($ilist->exists($id)) {
-            throw XAO::E::DO::Web::Order "add_items - item '$id' does not exist";
+            throw $self "add_items - item '$id' does not exist";
             $count++;
             next;
         }
@@ -440,8 +427,8 @@ sub new_item {
 
     my $self   = shift;
     my $args   = get_args(\@_);
-    my $oilist = $args->{list} || throw XAO::E::DO::Web::Order "new_item - no list argument";
-    my $item   = $args->{item} || throw XAO::E::DO::Web::Order "new_item - no item argument";
+    my $oilist = $args->{list} || throw $self "new_item - no list argument";
+    my $item   = $args->{item} || throw $self "new_item - no item argument";
 
     my ($oitem, $id) = ($oilist->get_new, '');
     foreach (sort $oitem->keys) {
@@ -462,7 +449,7 @@ sub new_item {
     }
 
     # attach ordered item with specified id
-    $id ? $oilist->put($id, $oitem) : throw XAO::E::DO::Web::Order "new_item - no item id";
+    $id ? $oilist->put($id, $oitem) : throw $self "new_item - no item id";
 
     #dprint "*** XAO::DO::Web::Order::new_item() STOP";
 
@@ -480,27 +467,28 @@ takes the following CGI Parameters:
 =cut
 
 sub set_shipto {
-
-    #dprint "*** XAO::DO::Web::Order::set_shipto() START";
-
-    my ($self, $rh_cgi) = @_;
+    my $self=shift;
+    my $rh_cgi=get_args(\@_);
 
     my $shipto_id = $rh_cgi->{shipto}
-                 || throw XAO::E::DO::Web::Order "set_shipto - no shipto argument";
+                 || throw $self "set_shipto - no shipto argument";
 
     # Note: get customer from clipboard - this requires IdentifyUser to have
     # run previously in 'check' mode!
     # XXX 'customer' is hardcoded for now
     my $customer = $self->_get_customer('customer')
-                || throw XAO::E::DO::Web::Order "set_shipto - no customer object";
+                || throw $self "set_shipto - no customer object";
     my $alist    = $customer->get('Addresses'); # XXX hardcoded for now
     my $address  = $alist->get($shipto_id);
-    my $order    = $self->_get_order || throw XAO::E::DO::Web::Order "set_shipto - no order";
+    my $order    = $self->_get_order || throw $self "set_shipto - no order";
     foreach ($address->keys) {
         my $rh_descr = $address->describe($_);
         next if $rh_descr->{type} eq 'key';
         $order->put("shipto_$_", $address->get($_));
     }
+    if($order->exists('shipto_id')) {
+        $order->put(shipto_id => $shipto_id);
+    }
     #dprint "    %% Shipto Address:";
     #foreach (sort $address->keys) { dprint "    %% * shipto_$_: ".$order->get("shipto_$_"); }
 
@@ -520,15 +508,13 @@ takes the following CGI Parameters:
 =cut
 
 sub set_shipmethod {
-
-    #dprint "*** XAO::DO::Web::Order::set_shipmethod() START";
-
-    my ($self, $rh_cgi) = @_;
+    my $self=shift;
+    my $rh_cgi=get_args(\@_);
 
     my $shipmethod_id = $rh_cgi->{shipmethod}
-                     || throw XAO::E::DO::Web::Order "set_shipmethod - no shipmethod argument";
+                     || throw $self "set_shipmethod - no shipmethod argument";
 
-    my $order = $self->_get_order || throw XAO::E::DO::Web::Order "set_shipmethod - no order";
+    my $order = $self->_get_order || throw $self "set_shipmethod - no order";
     $order->put('shipmethod', $shipmethod_id);
 
     #dprint "*** XAO::DO::Web::Order::set_shipmethod() STOP";
@@ -547,34 +533,33 @@ takes the following CGI Parameters:
 =cut
 
 sub set_paymethod {
+    my $self=shift;
+    my $rh_cgi=get_args(\@_);
 
-    #dprint "*** XAO::DO::Web::Order::set_paymethod() START";
+    my $paymethod_id = $rh_cgi->{paymethod} ||
+        throw $self "set_paymethod - no paymethod argument";
 
-    my ($self, $rh_cgi) = @_;
-
-    my $paymethod_id = $rh_cgi->{paymethod}
-                    || throw XAO::E::DO::Web::Order "set_paymethod - no paymethod argument";
-
-    # Note: get customer from clipboard - this requires IdentifyUser to have
-    # run previously in 'check' mode!
+    # Note: get customer from clipboard - this requires IdentifyUser to
+    # have run previously in 'check' mode!
     # XXX 'customer' is hardcoded for now
-    my $customer  = $self->_get_customer('customer')
-                 || throw XAO::E::DO::Web::Order "set_paymethod - no customer object";
+    my $customer  = $self->_get_customer('customer') ||
+        throw $self "set_paymethod - no customer object";
     my $pmlist    = $customer->get('PayMethods'); # XXX hardcoded for now
     my $paymethod = $pmlist->get($paymethod_id);
-    my $order     = $self->_get_order || throw XAO::E::DO::Web::Order "set_paymethod - no order";
+    my $order     = $self->_get_order ||
+        throw $self "set_paymethod - no order";
     foreach ($paymethod->keys) {
         my $rh_descr = $paymethod->describe($_);
         next if $rh_descr->{type} eq 'key';
         $order->put("pay_$_", $paymethod->get($_));
     }
-    #dprint "    %% Payment Method:";
-    #foreach (sort $paymethod->keys) { dprint "    %% * paymethod_$_: ".$order->get("paymethod_$_"); }
-
-    #dprint "*** XAO::DO::Web::Order::set_paymethod() STOP";
+    if($order->exists('pay_id')) {
+        $order->put(pay_id => $paymethod_id);
+    }
 
     return 1;
 }
+
 ###############################################################################
 
 =item calc_item_totals ($)
@@ -584,9 +569,8 @@ This method calculates and saves the item total.
 =cut
 
 sub calc_item_totals($) {
-    #dprint "*** XAO::DO::Web::Order::calc_item_totals() START";
     my $self   = shift;
-    my $order  = $self->_get_order || throw XAO::E::DO::Web::Order "calc_item_totals - no order";
+    my $order  = $self->_get_order || throw $self "calc_item_totals - no order";
     my $ilist  = $order->get('Products');
     my $ptotal = 0;
     foreach ($ilist->keys) {
@@ -597,22 +581,26 @@ sub calc_item_totals($) {
     #dprint "*** XAO::DO::Web::Order::calc_item_totals() STOP";
     return $order->put('total_items', $ptotal);
 }
+
 ###############################################################################
-# XXX This is the simplest possible case of this method. The 'price' property
-# is hardcoded and a possible sale price is not taken into consideration.
+# XXX This is the simplest possible case of this method. The 'price'
+# property is hardcoded and a possible sale price is not taken into
+# consideration.
+#
 sub order_price {
     my ($self, $item) = @_;
     return sprintf("%.2f", $item->get('price'));
 }
+
 ###############################################################################
 
 =item calc_shipping_totals ($)
 
-This method calculates and saves the shipping total.  Currently the calculation
-present is for sample purposes only (5% of items total).  Override this method
-with your custom shipping calculation.  Make sure that, in your custom version,
-you put the shipping total in the order's 'total_shipping' property and return
-the shipping total.
+This method calculates and saves the shipping total.  Currently the
+calculation present is for sample purposes only (5% of items total).
+Override this method with your custom shipping calculation.  Make sure
+that, in your custom version, you put the shipping total in the order's
+'total_shipping' property and return the shipping total.
 
 =cut
 
@@ -620,29 +608,30 @@ the shipping total.
 sub calc_shipping_totals($) {
     #dprint "*** XAO::DO::Web::Order::calc_shipping_totals() START";
     my $self   = shift;
-    my $order  = $self->_get_order || throw XAO::E::DO::Web::Order "calc_shipping_totals - no order";
+    my $order  = $self->_get_order || throw $self "calc_shipping_totals - no order";
     my $total_shipping = $order->get('shipto_state')
                        ? sprintf("%.2f", $order->get('total_items') * 0.05)
                        : '';
     #dprint "*** XAO::DO::Web::Order::calc_shipping_totals() STOP";
     return $order->put('total_shipping', $total_shipping);
 }
+
 ###############################################################################
 
 =item calc_tax_totals ($)
 
-This method calculates and saves the tax total. Currently the calculation
-present is for sample purposes only (8.25% of items total if shipping to
-California, USA).  Override this method with your custom tax calculation.
-Make sure that, in your custom version, you put the tax total in the order's
-'total_tax' property and return the tax total.
+This method calculates and saves the tax total. Currently the
+calculation present is for sample purposes only (8.25% of items total if
+shipping to California, USA).  Override this method with your custom tax
+calculation.  Make sure that, in your custom version, you put the tax
+total in the order's 'total_tax' property and return the tax total.
 
 =cut
 
 sub calc_tax_totals($) {
     #dprint "*** XAO::DO::Web::Order::calc_tax_totals() START";
     my $self   = shift;
-    my $order  = $self->_get_order || throw XAO::E::DO::Web::Order "calc_tax_totals - no order";
+    my $order  = $self->_get_order || throw $self "calc_tax_totals - no order";
     my $merchant_state = 'CA'; # XXX hardcoded for now
     return '0.00' unless $merchant_state eq $order->get('shipto_state'); # XXX hardcoded for now
     my $taxable_total  = $order->get('total_items') + $order->get('total_shipping');
@@ -650,6 +639,7 @@ sub calc_tax_totals($) {
     return $order->put('total_tax', $total_tax);
     #dprint "*** XAO::DO::Web::Order::calc_tax_totals() STOP";
 }
+
 ###############################################################################
 
 =item calc_grand_total ($)
@@ -661,7 +651,7 @@ This method calculates and saves the grand total.
 sub calc_grand_total($) {
     #dprint "*** XAO::DO::Web::Order::calc_grand_total() START";
     my $self   = shift;
-    my $order  = $self->_get_order || throw XAO::E::DO::Web::Order "calc_grand_total - no order";
+    my $order  = $self->_get_order || throw $self "calc_grand_total - no order";
     my $grand_total = sprintf(
                           "%.2f",
                           $order->get('total_items')
@@ -671,12 +661,12 @@ sub calc_grand_total($) {
     return $order->put('total_grand', $grand_total);
     #dprint "*** XAO::DO::Web::Order::calc_grand_total() STOP";
 }
+
 ###############################################################################
 #
 # Display page using all order properties (use all uppercase keys)
 #
 sub _display {
-    #dprint "*** XAO::DO::Web::Order::_display() START";
     my $self = shift;
     my $args = get_args(\@_);
     my %pass;
@@ -699,9 +689,9 @@ sub _display {
     }
     my $page = $self->object;
     $page->display(\%pass);
-    #dprint "*** XAO::DO::Web::Order::_display() STOP";
     return 1;
 }
+
 ###############################################################################
 #
 # Gets order from clipboard or database. Returns order object or empty string.
@@ -749,7 +739,7 @@ sub _get_order($;$) {
         # Try to get order id from cookie and then try to get order
         #
         my $cookie_name = $oconfig->{order_cookie}
-                       || throw XAO::E::DO::Web::Order "_get_order - no order cookie name in config";
+                       || throw $self "_get_order - no order cookie name in config";
         $order_id = $sconfig->cgi->cookie(-name => $cookie_name);
         #dprint "    %% No Order ID (early return)" unless $order_id;
         return '' unless $order_id;
@@ -767,6 +757,7 @@ sub _get_order($;$) {
 
     return $order;
 }
+
 ###############################################################################
 #
 # Creates a new order, adds it to the order list and finally sets an order
@@ -804,6 +795,7 @@ sub _create_order($;$) {
 
     return $order;
 }
+
 ###############################################################################
 #
 # Return customer as retrieved from clipboard - this requires IdentifyUser
@@ -821,14 +813,16 @@ sub _get_customer {
     # Get clipboard uri for requested type of user
     # XXX shouldn't IdentifyUser.pm provide a method for this?
     my $idconfig = $self->siteconfig->get('identify_user')
-                || throw XAO::E::DO::Web::Order "_get_customer - no 'identify_user' config";
+                || throw $self "_get_customer - no 'identify_user' config";
     $idconfig    = $idconfig->{$type}
-                || throw XAO::E::DO::Web::Order "_get_customer - no 'identify_user' config for '$type'";
+                || throw $self "_get_customer - no 'identify_user' config for '$type'";
     my $cb_uri = $idconfig->{cb_uri} || "/IdentifyUser/$type";
 
     return $self->clipboard->get("$cb_uri/object");
 }
+
 ###############################################################################
+
 sub form_fields {
     my $self=shift;
     return [
@@ -1004,7 +998,9 @@ sub form_fields {
        #},
     ];
 }
+
 ###############################################################################
+
 1;
 __END__
 
